//
//  MKVI Monorail Demo
//
//  This is a test script to control multiple trains stopping at each
//  side of the station during a complete circuit of the beamway.  The
//  script contains examples of simple message handling for multiple
//  trains simultaneously, controlled animation, sound effects, various
//  camera options, and the simple interactions between trains needed to
//  keep them running indeffinately.  It does not yet contain any switching
//  or signaling.
//
//  Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//      Users may modify this scenario or use it as a basis for their own
//      work, but should not redistribute it without including this original
//      copyright notice.
//
//  This is the version developed for the the original Service Pack 3 (SP3)
//  version of trains, and does not run on newer builds that have different
//  timing requirements for train creation. See the scenario named 
//  MKVIMonorailDemo.gs for newer systems.

include "interface.gs"
include "gs.gs"
include "TrackMark.gs"
include "Train.gs"
include "KUID.gs"
include "world.gs"
include "junction.gs"
include "Trigger.gs"
include "Signal.gs"

//===========================================================================//
class TrainData
//
//   Class:      TrainData
//   Purpose:    This is a class to hold train informaiton that we cannot hold
//                  in Train objects.  If we use an array of trains, we can also
//                  use an array of TrainData objects to describe them.  At some
//                  point, we should be able to store this object within each train.
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      06/25/02    Started.

{
//  Constant Values:  NONE

//  Protected Instance Attributes

    int     m_nID            = 0;       // the ID value of this train
    bool    m_bInStation     = false;   // this train is currently in a station flag
    string  m_strName        = "";      // the name of this train
    string  m_strLastStation = "";      // the name of the last station visited by this train

//---------------------------------------------------------------------------//
//  Value setting routines

	public void SetID           (int nID)
    	{m_nID            = nID;}
	public void SetIsInStation  (bool bInStation)
    	{m_bInStation     = bInStation;}
	public void SetName         (string strName)
    	{m_strName        = strName;}
	public void SetLastStation  (string strLastStation)
    	{m_strLastStation = strLastStation;}

//---------------------------------------------------------------------------//
//  Value getting routines

	public int      GetID ()
    	{return m_nID;}
	public bool     GetIsInStation ()
    	{return m_bInStation;}
	public string   GetName ()
    	{return m_strName;}
	public string   GetLastStation ()
    	{return m_strLastStation;}
};
//  End of class TrainData
//===========================================================================//



//===========================================================================//
class StationData
//
//   Class:      StationData
//   Purpose:    This is a class to hold station informaiton that we cannot hold
//                  in any other objects.  With only one station in the Demo,
//                  we need only one of these objects.
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      09/01/02    Started.

{
//  Constant Values:  NONE

//  Protected Instance Attributes

    int     m_nTrainsInStation = 0;     // the number of trains in the station
    int     m_nFirstTrainIn    = 0;     // the index of the first train to arrive
    int     m_nSecondTrainIn   = 0;     // the index of the second train to arrive

//---------------------------------------------------------------------------//
//  Value setting routines

	public void SetTrainsInStation  (int nTrainsInStation)
    	{m_nTrainsInStation     = nTrainsInStation;}
	public void SetFirstTrainIn     (int nFirstTrainIn)
    	{m_nFirstTrainIn        = nFirstTrainIn;}
	public void SetSecondTrainIn     (int nSecondTrainIn)
    	{m_nSecondTrainIn       = nSecondTrainIn;}

//---------------------------------------------------------------------------//
//  Value getting routines

	public int      GetTrainsInStation ()
    	{return m_nTrainsInStation;}
	public int      GetFirstTrainIn ()
    	{return m_nFirstTrainIn;}
	public int      GetSecondTrainIn ()
    	{return m_nSecondTrainIn;}
};
//  End of class StationData
//===========================================================================//



//===========================================================================//
game class MKVIDemo isclass GameObject
//
//   Class:      MKVIDemo
//   Purpose:    This is the initial GameObject class used to control the MKVIDemo Scenario
//   Copyright (C) 2002 Jack Emmerichs, Milwaukee Wisconsin, 2002.
//   Created by: Jack Emmerichs
//
//   Change History---------
//      06/15/02    Started.

{
//  Constant Values

    define public bool  OPEN_DOORS     = true;  // Open the train doors
    define public bool  CLOSE_DOORS    = false; // Close the train doors

    define public int   TRAIN_COUNT    = 3;     // the number of trains in the simulation
    define public int   GREEN          = 0;     // Index of the green train in aTrain
    define public int   BLUE           = 1;     // Index of the blue train in aTrain
    define public int   RED            = 2;     // Index of the red train in aTrain

    define public float SPEED_BRAKE    = -1.00; // speed used to brake for the stations.
    define public float SPEED_STOPPED  = 0.00;  // speed used to stop the train
    define public float SPEED_STATION  = 0.30;  // speed within station areas
    define public float SPEED_FULL     = 1.00;  // speed used to get moving quickly

    define public string NORTH_STATION_NAME    = "LAKESIDE NORTH";
    define public string SOUTH_STATION_NAME    = "LAKESIDE SOUTH";

    public string [] aAnimName  = new string[0];
    public string [] aDoorName  = new string[0];
    
//  Protected Instance Attributes

    Train     [] aTrain         = new Train     [0];    // The array of trains we will be running
    TrainData [] aTrainData     = new TrainData [0];    // The array of TrainData objects describing
                                                        //    the aTrain objects.
    StationData  obStationData  = new StationData ();   // The data describing the demo station

    bool    bInStartup          = true;     // "scenario is starting up" flag
    bool    bLogging            = true;     // set true to enable logging
    bool    bAnimating          = true;     // set true to animate doors.

//  Forward references for Protected methods

    void AnimateTheDoors (Train     theTrain,
                          string    strTrainName,
                          int       nTrainIndex,
                          bool      bIsOpen);

//  Protected methods

//---------------------------------------------------------------------------//
  void Init(void)

// Descr:   Initialize the JfeText class by setting up the random number generator
// Params:  None.
// Returns: Nothing.

  {
    aAnimName[0] = "DoorAnimGreen.kin";
    aAnimName[1] = "DoorAnimBlue.kin";
    aAnimName[2] = "DoorAnimRed.kin";

    aDoorName[0] = "LeftDoorsGreen_panto.pm";
    aDoorName[1] = "LeftDoorsBlue_panto.pm";
    aDoorName[2] = "LeftDoorsRed_panto.pm";
  }


//---------------------------------------------------------------------------//
  void LoadTheMap(void)

//  Descr:   Load the map called "SCENARIO".  If it is missing, this activity
//           will die a horrible death.  Find the map, log the map's name in
//           the log file, and then load the map.
//  Params:  None.
//  Returns: Nothing.

  {
    KUID myMap;     // The map we are loading

    if (bLogging)
        Interface.Log ("===  Start of Scenario MKVIDemo ===");
    myMap = World.FindKUID("MKVIDemoLayout");
    if(!World.LoadMap(myMap))
    {
      Interface.Log ("Scenario MKVIDemo: Could not load Monorail Test 2");
      return;
    }
    if (bLogging)
        Interface.Log ("---    Loaded map '" + myMap.GetName() + "'");
//
//  Give the map time to load.
//  This is a hack that will be taken out later.
//
    Sleep(8.0f);
  }


//---------------------------------------------------------------------------//
  void LoadTheTrains(void)

// Descr:   Create the trains used in the scenario and place them on the layout.
// Params:  None.
// Returns: Nothing.

  {
    KUID[]      consist = new KUID[0];  // A consist used to build the trains.
    TrackMark   startMark;              // Working track mark for each train placement
    Vehicle[]   vehicles;               // the vehicles that make up a single train
    int         nTrainIndex;            // Index of the train we are building.
    int         nCarIndex;              // Index of the vehicles in a train.
    int         hAnimation;             // Animation handle from added door animation

//  Build each of the trains with the proper cars on the proper track mark

    if (bLogging)
        Interface.Log ("-- Start loading the trains");
    Sleep(3.0f);
    for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
    {
//  Build the TrainData description objects and build the trains with the right cars,
//  place them on the right track marks, and set their individual attributes.

        if (bLogging)
            Interface.Log ("-- Starting on train " + nTrainIndex);
        aTrainData[nTrainIndex] = new TrainData ();

//  Build the parts of each train that are unique

        switch (nTrainIndex)
        {
            case 0:     // Monorail Green at Lakeside North
                consist[0] = World.FindKUID("MKVICabGreen");
                consist[1] = World.FindKUID("MKVICarGreen");
                consist[5] = World.FindKUID("MKVIReverseCabGreen");
                startMark = cast<TrackMark>(Router.GetGameObject("StationNorthMark"));
                aTrainData[nTrainIndex].SetName ("MONORAIL GREEN");
                break;
            case 1:     // Monorail Blue at Lakeside South
                consist[0] = World.FindKUID("MKVICabBlue");
                consist[1] = World.FindKUID("MKVICarBlue");
                consist[5] = World.FindKUID("MKVIReverseCabBlue");
                startMark = cast<TrackMark>(Router.GetGameObject("StationSouthMark"));
                aTrainData[nTrainIndex].SetName ("MONORAIL BLUE");
                break;
            case 2:     // Monorail Red at NortheastCurve
                consist[0] = World.FindKUID("MKVICabRed");
                consist[1] = World.FindKUID("MKVICarRed");
                consist[5] = World.FindKUID("MKVIReverseCabRed");
                startMark = cast<TrackMark>(Router.GetGameObject("RedStart"));
                aTrainData[nTrainIndex].SetName ("MONORAIL RED");
                break;
            default:
                break;
        }
//  Set the remaining cars to match the first car, and then build the train.

        consist[2] = consist[1];
        consist[3] = consist[1];
        consist[4] = consist[1];
        aTrain[nTrainIndex] = World.CreateTrain(consist, startMark, true);
        Sleep(1.0f);

//  Set the status and configuration of the train.

        if (bLogging)
            Interface.Log ("---- configuring the train ");
        aTrain[nTrainIndex].SetPhysicsMode (Train.PHYSICS_DCC);
        aTrain[nTrainIndex].SetAutopilotMode (Train.CONTROL_MANUAL);
        aTrainData[nTrainIndex].SetID (aTrain[nTrainIndex].GetId());
        aTrainData[nTrainIndex].SetIsInStation (false);
        Sleep(1.0f);
        
//  If we are animating the doors, add the doors to the vehicles

        if (bAnimating)
        {
            vehicles = aTrain[nTrainIndex].GetVehicles();
            for (nCarIndex = 0; nCarIndex < vehicles.size(); nCarIndex++)
            {
                hAnimation = vehicles[nCarIndex].AddAnimatedObject("a.pant0", aDoorName[nTrainIndex], aAnimName[nTrainIndex]);
                if (hAnimation == -1)
                {
                    if (bLogging)
                        Interface.Log ("-- AddAnimation for vehicle [" + nCarIndex +
                                       "] failed, animation skipped.");
                }
                else
                {
                    if (bLogging)
                        Interface.Log ("-- Vehicle [" + nCarIndex +
                                       "] has an animation handle of: [" + hAnimation + "].");
                }
            }
        }

//  Set the cameras and log the end of the train setup
        
        World.SetCamera(aTrain[nTrainIndex], World.CAMERA_EXTERNAL);
        Sleep(1.0f);
        World.SetCameraAngle((int)240, (int)-10, (float)25);
        Sleep(1.0f);
        if (bLogging)
            Interface.Log ("-- Finished train " + nTrainIndex);
        Sleep(1.0f);
    }
//  Put the camera on Monorail green, watch it come out of the station, and
//  then move to the tracking cameras, get the trains moving, and play the horn
//  to start the action.

    World.SetCamera(aTrain[GREEN], World.CAMERA_EXTERNAL);
    World.SetCameraAngle((int)240, (int)-10, (float)25);
    Sleep(5.0f);
    for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
    {
        aTrain[nTrainIndex].SetDCCThrottle (SPEED_FULL);
    }
//      vehicles = aTrain[GREEN].GetVehicles();
//      World.PlaySound ("air_horn_3.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
    World.PlaySound ("air_horn_3.wav", 0.75, 0, 200, null, null);
    World.SetCameraAngle((int)240, (int)-10, (float)25);
    PostMessage(aTrain[GREEN], "Trigger", "UseTrackingCamera", 12);   // debug
  }


//---------------------------------------------------------------------------//
  void handleTriggerMessages (Message msg)

// Descr:   Handle messages received by the trains.  Messages include those
//          sent by triggers and those we send to the trains.
// Params:  msg:     The message we are to process.
// Returns: Nothing.

  {
    Train       theTrain;               // the train we are controlling
    Trigger     theTrigger;             // the trigger we have just entered
    Vehicle[]   vehicles;               // the vehicles that make up a single train
    int         nTestIndex;             // index used to test which train we are using
    int         nTrainIndex;            // Index of the train we are handling.
    float       fSpeed = 0.0;           // the speed indicated by a speed trigger
    string      strTrainName;           // the name of the train for this message
    string      strTriggerName;         // the name of the trigger for this message
    string      strStationName;         // used to check the name of the station
    string      strWork;                // a general purpose work string

//  Log the message

    if (bLogging)
        Interface.Log ("Demo::handleTriggerMessages msg.major=" + msg.major +
                       " msg.minor=" + msg.minor +
                       " msg.src=" + (cast<GameObject> msg.src).GetName() +
                       " msg.dst=" + (cast<GameObject> msg.dst).GetId());

//  Handle the various message types.

    if (    (msg.minor == "Leave")
        or  (bInStartup)
       )
    {
//  Received a message from a trigger we are leaving, or we are still starting up the
//  scenario.  We don't process the Leave message (or anything during startup), so 
//  log this fact, and ignore it.

        if (bLogging)
            Interface.Log ("Leave is not a useful message, or starting up.  exiting now.");
        return;
    }
//  Received a message to process.  Get the train and the trigger we are using.
//  Also, get the trigger name and the train array index (which provides the train name).

    theTrain   = (cast<Train> msg.dst);
    theTrigger = (cast<Trigger> msg.src);
    strTriggerName = (cast<GameObject> msg.src).GetName();

    for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
    {
        if (theTrain.GetId() == aTrainData[nTestIndex].GetID())
        {
//  Found this train in the array, save the index and get the train name.
        
            nTrainIndex = nTestIndex;
            strTrainName = aTrainData[nTrainIndex].GetName();
            break;
        }
    }

    if (msg.minor == "UseTrackingCamera")
    {
//  Set the view back to the tracking cameras.

        World.SetCamera(theTrain, World.CAMERA_TRACKING);
        return;
    }
    
    if (msg.minor == "IgnoreStation")
    {
//  Find the train not in the station, and put the tracking cameras on it
//  (watching trains sitting in the station is pretty boreing).

        for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
        {
            if (! aTrainData[nTestIndex].GetIsInStation())  // for the train not in the station
            {
                World.SetCamera(aTrain[nTestIndex], World.CAMERA_TRACKING);
                if (bLogging)
                    Interface.Log ("Ignore station, setting the camer on: " + aTrainData[nTestIndex].GetName());
            }
        }
        return;
    }


    if (msg.minor == "Stopped")
    {

//  A train just stopped moving.  If it is at the station (there should be no other
//  cases), open the doors.  If it is the first train into the station, play the
//  door opening sound file.

        if (aTrainData[nTestIndex].GetIsInStation())
        {
            if (bAnimating)
                AnimateTheDoors (theTrain, strTrainName, nTrainIndex, OPEN_DOORS);
            if (obStationData.GetFirstTrainIn() == nTrainIndex)        // first train to arrive
            {
                if (bLogging)
                    Interface.Log ("Playing Entry .wav file for " + strTrainName + ".");
//                vehicles = theTrain.GetVehicles();
//                World.PlaySound ("OpeningMessage.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
//                World.PlaySound ("Door Open wdw monorail.wav", 0.70, 0, 50, vehicles[0], "a.bog0");
                World.PlaySound ("OpeningMessage.wav", 0.70, 0, 50, null, null);
            }
        }
        return;
    }


    if (msg.minor == "StartTrains")
    {
//  For each train in the station, close its doors, record the fact that it is
//  leaving, start it moving, and mark it no longer in the station.

        for (nTestIndex = 0; nTestIndex < TRAIN_COUNT; nTestIndex++)
        {
            if (aTrainData[nTestIndex].GetIsInStation())
            {
                strTrainName = aTrainData[nTestIndex].GetName();
                if (bAnimating)
                    AnimateTheDoors (aTrain[nTestIndex], strTrainName, nTrainIndex, CLOSE_DOORS);
                if (bLogging)
                    Interface.Log ("Restarting " + strTrainName + " at " +
                                    aTrainData[nTestIndex].GetLastStation() + " now.");
                Interface.Print(strTrainName + " LEAVING " +
                                    aTrainData[nTestIndex].GetLastStation());
                Interface.SetObjective(strTrainName + " now leaving: " +
                                        aTrainData[nTestIndex].GetLastStation(), "");
                Interface.ShowObjective(true);
                aTrain[nTestIndex].SetDCCThrottle (SPEED_FULL);
                aTrainData[nTestIndex].SetIsInStation (false);
            }
        }

//  All trains have now left the station.  Clear the station count, post the message
//  to go back to the tracking cameras in a few seconds, and play the air-horn sound

        obStationData.SetTrainsInStation(0);
        PostMessage(aTrain[GREEN], "Trigger", "UseTrackingCamera", 11);
//      vehicles = aTrain[GREEN].GetVehicles();
//      World.PlaySound ("air_horn_3.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
        World.PlaySound ("air_horn_3.wav", 0.75, 0, 200, null, null);
        if (bLogging)
            Interface.Log ("Playing air_horn_3.wav.");
        return;
    }
    

    else if (msg.minor == "LeaveCabAtStation")  // change back from the cab view
    {

//  Move the camera back to the second train into the station.

        if (bLogging)
            Interface.Log ("Handling LeaveCabAtStation message.");
        World.SetCamera(aTrain[obStationData.GetSecondTrainIn()], World.CAMERA_EXTERNAL);
        World.SetCameraAngle((int)240, (int)-10, (float)25);
    }



    if (msg.minor == "Enter")
    {
//  We just entered a trigger.  Process this message depending on the trigger name

        strStationName = strTriggerName;
        Str.Right (strStationName, 5);


        if (strTriggerName[0,5] == "Speed")
        {
            fSpeed = Str.ToFloat(strTriggerName[6,]) / 100;
            theTrain.SetDCCThrottle (fSpeed);
            if (bLogging)
                Interface.Log ("For trigger name [" + strTriggerName + "], Setting " + strTrainName + " speed to: " + fSpeed);
        }

        else if (strTriggerName == "StartInCab") // Start viewing from inside the cab for Green
        {
            if (nTrainIndex == GREEN)
                World.SetCamera(theTrain, World.CAMERA_INTERNAL);
        }

        else if (strTriggerName == "EndInCab") // End viewing from inside the cab for Green
        {
            if (nTrainIndex == GREEN)
                World.SetCamera(theTrain, World.CAMERA_TRACKING);
        }

        else if (strTriggerName[0,5] == "Brake")
        {
//  Breaking for an upcoming station.  Log that, and brake

            if (obStationData.GetTrainsInStation() < 1)                 // first train to arrive
            {
                obStationData.SetTrainsInStation(1);
                obStationData.SetFirstTrainIn(nTrainIndex);
                if (nTrainIndex == GREEN)                               // if green, drive into the station.
                    World.SetCamera(theTrain, World.CAMERA_INTERNAL);
            }
            else                                                        // second train to arrive
            {
                obStationData.SetTrainsInStation(2);
                obStationData.SetSecondTrainIn(nTrainIndex);
            }
            if (bLogging)
                Interface.Log ("Breaking " + strTrainName + " now.");
            theTrain.SetDCCThrottle (SPEED_BRAKE);
        }

        else if (strTriggerName[0,8] == "Approach")
        {
//  Approaching a station.  Log it and slow the train down.  If this is
//  the second train to arrive, put the camera on the train.

            aTrainData[nTrainIndex].SetIsInStation (true);
            if (strStationName == "North")
            {
                strTriggerName = NORTH_STATION_NAME;
            }
            else
            {
                strTriggerName = SOUTH_STATION_NAME;
            }
            
            if (    (obStationData.GetSecondTrainIn() == nTrainIndex)   // second train to arrive
                and (obStationData.GetFirstTrainIn() != GREEN))         // and green is not "driving"
            {
                World.SetCamera(theTrain, World.CAMERA_EXTERNAL);       // follow this train with the camera
                World.SetCameraAngle((int)240, (int)-10, (float)25);
            }

            if (bLogging)
                Interface.Log ("Slowing " + strTrainName + " at " + strTriggerName + " now.");
            Interface.Print(strTrainName + " APPROACHING " + strTriggerName);
            theTrain.SetDCCThrottle (SPEED_STATION);
        }


        else if (strTriggerName[0,7] == "Station")
        {
//  Arrived at a station.  Log it, and stop the train.  Also, if this is the last
//  (second) train to arrive, post a message to ignore the station after another
//  few seconds as it's pretty boring.  It doesn't matter who you send it to as
//  we'll simply catch it and then go look at the proper train.

            if (strStationName == "North")
            {
                strTriggerName = NORTH_STATION_NAME;
            }
            else
            {
                strTriggerName = SOUTH_STATION_NAME;
            }
            if (obStationData.GetSecondTrainIn() == nTrainIndex)        // second train to arrive
            {
                PostMessage(theTrain, "Trigger", "IgnoreStation", 9);
                if (bLogging)
                    Interface.Log ("Posting IgnoreStation message.");
            }
            else
            {
                if (nTrainIndex == GREEN)                               // if green, about to leave driver seat
                {
                    if (bLogging)
                        Interface.Log ("Posting LeaveCabAtStation message.");
                    PostMessage(aTrain[GREEN], "Trigger", "LeaveCabAtStation", 2);
                }
            }

            if (bLogging)
                Interface.Log ("Stopping " + strTrainName + " at " + strTriggerName + " now.");
            Interface.Print(strTrainName + " NOW AT " + strTriggerName);
            Interface.SetObjective(strTrainName + " has arrived at: " + strTriggerName, "");
            Interface.ShowObjective(true);
            theTrain.SetDCCThrottle (SPEED_STOPPED);

//  Set the current stop for the train and indicate it is now in the station.

            aTrainData[nTrainIndex].SetLastStation (strTriggerName);
        }


        else if (strTriggerName == "EmptyStation")  // leaving the station
        {
            Train   WatchedTrain;       // the train to watch as it leaves
            string  strTrainName;       // the name of the train being restarted

//  The free-running train has tripped the trigger to restart the trains in the
//  station.  Start the Close doors sound and put the camera on the proper train.

            if (bLogging)
                Interface.Log ("Playing Door Close jack wagner monorail.wav.");

            if (obStationData.GetFirstTrainIn() == GREEN)   // watch green train
                WatchedTrain = aTrain[GREEN];
            else                                        // or second train in
                WatchedTrain = aTrain[obStationData.GetSecondTrainIn()];
//            vehicles = WatchedTrain.GetVehicles();
//            World.PlaySound ("ClosingMessage.wav", 10000.0, 200.0, 6000.0, vehicles[0], "a.limfront");
            World.PlaySound ("ClosingMessage.wav", 0.70, 0, 50, null, null);
            World.SetCamera(WatchedTrain, World.CAMERA_EXTERNAL);
            World.SetCameraAngle((int)240, (int)-10, (float)25);
            PostMessage(aTrain[GREEN], "Trigger", "StartTrains", 2.5f);
        }
    }
  }


//---------------------------------------------------------------------------//
  void AnimateTheDoors (Train   theTrain,
                        string  strTrainName,
                        int     nTrainIndex,
                        bool    bIsOpen)

// Descr:   Open or close all the doors of a train.
// Params:  theTrain        // The train whose doors we are going to animate
//          strTrainName    // The name of the train to animate
//          bIsOpen         // Indicates whether to open or close the doors.
// Returns: Nothing.

  {
    Vehicle[]   vehicles = theTrain.GetVehicles();
    string      m_strMsg = "close";
    int         nCarIndex;
    int         hAnimation;
    if (bIsOpen)
        m_strMsg = "open";

//  Log the animation we are about to try

    Interface.Log ("Try to [" + m_strMsg + "]the doors on [" + strTrainName + "].");
    for (nCarIndex = 0; nCarIndex < vehicles.size(); nCarIndex++)
    {
        hAnimation = vehicles[nCarIndex].GetAnimatedObject("a.pant0");
        if (hAnimation == -1)
        {
            if (bLogging)
                Interface.Log ("-- GetAnimation for vehicle [" + nCarIndex +
                               "] has no animation handle: [" + hAnimation + "], trying AddAnimatedObject.");
            hAnimation = vehicles[nCarIndex].AddAnimatedObject("a.pant0", aDoorName[nTrainIndex], aAnimName[nTrainIndex]);
            if (hAnimation == -1)
            {
                if (bLogging)
                    Interface.Log ("-- AddAnimation for vehicle [" + nCarIndex +
                                   "] has no animation handle: [" + hAnimation + "], animation skipped.");
                return;
            }
        }
        if (bLogging)
            Interface.Log ("-- Vehicle [" + nCarIndex +
                           "] has an animation handle of: [" + hAnimation + "].");
        vehicles[nCarIndex].PlayDirection(hAnimation, bIsOpen);
    }
  }


//---------------------------------------------------------------------------//
	thread void main(void)

// Descr:   This is the script's main loop that kicks off all the other processing
// Params:  None
// Returns: Nothing.

  {
    int nTrainIndex;            // Index of the train we are building.
//  Log the start of the script

    Interface.Print("!!!! Start of MKVIDemo.gs. !!!!");
    if (bLogging)
        Interface.Log ("Loading map for MKVIDemo.gs.");

//  Load the map and put the trains on it.  Also, display the on-screen help

    Init ();
    LoadTheMap();
    LoadTheTrains();
    Interface.ShowOnScreenHelp(false);
    Interface.ShowObjective(true);

//  Define the message handlers for all three trains.

    for (nTrainIndex = 0; nTrainIndex < TRAIN_COUNT; nTrainIndex++)
    {
        AddHandler(aTrain[nTrainIndex], "Trigger", (string) null, "handleTriggerMessages");
    }

//  Log the fact that everything is up and running

    if (bLogging)
        Interface.Log ("Activity ready.");
    Interface.SetObjective("Running...", "");

    Sleep (5);
    bInStartup = false;
  }
};


